package com.tzld.longarticle.recommend.server.model.entity.crawler;

import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.NoArgsConstructor;

import javax.persistence.*;
import java.io.Serializable;
import java.util.Date;

@Data
@AllArgsConstructor
@NoArgsConstructor
@Entity
@Table(name = "long_articles_video")
@IdClass(com.tzld.longarticle.recommend.server.model.entity.crawler.LongArticlesVideo.PK.class)
public class LongArticlesVideo implements Serializable {

    @Id
    @GeneratedValue(strategy = GenerationType.IDENTITY)
    private Integer id;

    /**
     * 请求唯一最终 id
     */
    @Id
    private String traceId;

    /**
     * 文章id
     */
    @Column(name = "content_id", length = 64, columnDefinition = "varchar(64) DEFAULT NULL COMMENT '文章id'")
    private String contentId;

    /**
     * 公众号 id
     */
    @Column(name = "gh_id", length = 255, columnDefinition = "varchar(255) CHARACTER SET utf8 DEFAULT NULL COMMENT '公众号 id'")
    private String ghId;

    /**
     * 公众号名称
     */
    @Column(name = "account_name", length = 255, columnDefinition = "varchar(255) DEFAULT NULL COMMENT '公众号名称'")
    private String accountName;

    /**
     * 文章标题
     */
    @Column(name = "article_title", length = 255, columnDefinition = "varchar(255) DEFAULT NULL COMMENT '文章标题'")
    private String articleTitle;

    /**
     * 文章正文
     */
    @Lob
    @Column(name = "article_text", columnDefinition = "text COMMENT '文章正文'")
    private String articleText;

    /**
     * 内容状态
     * 0: 未处理
     * 1: 处理中
     * 2: 处理完成
     */
    @Column(name = "content_status", nullable = false, columnDefinition = "int(1) DEFAULT '0' COMMENT '0: 未处理， 1: 处理中， 2: 处理完成'")
    private Integer contentStatus = 0;

    /**
     * kimi生成的符合小程序的标题
     */
    @Column(name = "kimi_title", length = 255, columnDefinition = "varchar(255) DEFAULT NULL COMMENT 'kimi生成的符合小程序的标题'")
    private String kimiTitle;

    /**
     * kimi 通过文本生成的总结性文本
     */
    @Column(name = "kimi_summary", length = 255, columnDefinition = "varchar(255) DEFAULT NULL COMMENT 'kimi 通过文本生成的总结性文本'")
    private String kimiSummary;

    /**
     * kimi 通过文本生成的总结性搜索关键词
     */
    @Column(name = "kimi_keys", length = 255, columnDefinition = "varchar(255) DEFAULT NULL COMMENT 'kimi 通过文本生成的总结性搜索关键词'")
    private String kimiKeys;

    /**
     * 召回 id1
     */
    @Column(name = "recall_video_id1", columnDefinition = "int(11) DEFAULT NULL COMMENT '召回 id1'")
    private Integer recallVideoId1;

    /**
     * 召回 id2
     */
    @Column(name = "recall_video_id2", columnDefinition = "int(11) DEFAULT NULL COMMENT '召回 id2'")
    private Integer recallVideoId2;

    /**
     * 召回 id3
     */
    @Column(name = "recall_video_id3", columnDefinition = "int(11) DEFAULT NULL COMMENT '召回 id3'")
    private Integer recallVideoId3;

    /**
     * 返回结果 1
     */
    @Lob
    @Column(name = "result1", columnDefinition = "text COMMENT '返回结果 1'")
    private String result1;

    /**
     * 返回结果 2
     */
    @Lob
    @Column(name = "result2", columnDefinition = "text COMMENT '返回结果 2'")
    private String result2;

    /**
     * 返回结果 3
     */
    @Lob
    @Column(name = "result3", columnDefinition = "text COMMENT '返回结果 3'")
    private String result3;

    /**
     * 是否成功, 1表示成功
     */
    @Column(name = "success", nullable = false, columnDefinition = "tinyint(1) DEFAULT '0' COMMENT '是否成功'")
    private Boolean success = false;

    /**
     * 更新时间
     */
    @Column(name = "update_time", nullable = false, columnDefinition = "datetime DEFAULT CURRENT_TIMESTAMP ON UPDATE CURRENT_TIMESTAMP COMMENT '更新时间'")
    private Date updateTime;

    /**
     * 请求时间戳
     */
    @Column(name = "request_time_stamp", columnDefinition = "int(16) DEFAULT NULL COMMENT '请求时间戳'")
    private Long requestTimeStamp;

    /**
     * 执行次数
     */
    @Column(name = "process_times", nullable = false, columnDefinition = "int(1) DEFAULT '0' COMMENT '执行次数'")
    private Integer processTimes = 0;


    @Data
    public static class PK implements Serializable {

        @Column(name = "id")
        private Integer id;
        @Column(name = "trace_id")
        private String traceId;

        public PK() {
        }


    }
}
