package com.tzld.longarticle.recommend.server.mq;


import com.alibaba.fastjson.JSONObject;
import com.aliyun.openservices.ons.api.Action;
import com.aliyun.openservices.ons.api.ConsumeContext;
import com.aliyun.openservices.ons.api.Message;
import com.aliyun.openservices.ons.api.MessageListener;
import com.tzld.longarticle.recommend.server.mapper.crawler.PushMessageCallbackMapper;
import com.tzld.longarticle.recommend.server.mapper.growth.NewPushMessageCallbackMapper;
import com.tzld.longarticle.recommend.server.model.bo.ReplyInfo;
import com.tzld.longarticle.recommend.server.model.param.CallbackParam;
import com.tzld.longarticle.recommend.server.repository.model.PushMessageCallback;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

import java.util.ArrayList;
import java.util.List;

@Slf4j
@Component
public class MessageCallbackCustomer implements MessageListener {


    @Autowired
    private PushMessageCallbackMapper pushMessageCallbackMapper;

    @Autowired
    private NewPushMessageCallbackMapper newPushMessageCallbackMapper;

    @Override
    public Action consume(Message message, ConsumeContext consumeContext) {
        CallbackParam param = JSONObject.parseObject(new String(message.getBody()), CallbackParam.class);
        log.info("param = {}", param);
        if (CollectionUtils.isEmpty(param.getReplyInfo())) {
            log.error("CallbackParam replyInfo is empty {}", param);
        }
        PushMessageCallback pushMessageCallback = new PushMessageCallback();
        pushMessageCallback.setGhId(param.getGhId());
        pushMessageCallback.setTimestamp(param.getTimestamp());
        pushMessageCallback.setOpenId(param.getOpenId());
        List<PushMessageCallback> insertList = new ArrayList<>();
        for (ReplyInfo replyInfo : param.getReplyInfo()) {
            PushMessageCallback insertPushMessageCallback = new PushMessageCallback();
            BeanUtils.copyProperties(pushMessageCallback, insertPushMessageCallback);
            insertPushMessageCallback.setMsgType(replyInfo.getMsgType());
            insertPushMessageCallback.setVideoId(replyInfo.getMiniVideoId());
            insertList.add(insertPushMessageCallback);
        }
        try {
            newPushMessageCallbackMapper.insertList(insertList);
        } catch (Exception e) {
            log.error("PushMessageCallback newPushMessageCallbackMapper insert pushMessageCallback={}, error={}", pushMessageCallback, e.getMessage());
            return Action.ReconsumeLater;
        }
        return Action.CommitMessage;
    }
}
