package com.tzld.longarticle.recommend.server.remote;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.tzld.longarticle.recommend.server.common.HttpPoolFactory;
import com.tzld.longarticle.recommend.server.model.vo.ProduceContentCrawlerVO;
import lombok.extern.slf4j.Slf4j;
import org.apache.http.HttpEntity;
import org.apache.http.StatusLine;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.entity.StringEntity;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.util.EntityUtils;
import org.springframework.stereotype.Service;

import java.nio.charset.StandardCharsets;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;

@Service
@Slf4j
public class CrawlerContentByPlanService {
    private final CloseableHttpClient client = HttpPoolFactory.aigcPool();
    private static final String url = "http://aigc-api.cybertogether.net/aigc/produce/content/getContentByCrawlerPlan";


    public List<ProduceContentCrawlerVO> getCrawlerContentByPlan(String crawlerPlanId, List<String> producePlanIds) {
        long start = System.currentTimeMillis();
        List<ProduceContentCrawlerVO> result = new ArrayList<>();
        JSONObject bodyParam = new JSONObject();
        JSONObject bodyParamParams = new JSONObject();
        bodyParamParams.put("crawlerPlanId", crawlerPlanId);
        bodyParamParams.put("producePlanIds", producePlanIds);
        bodyParam.put("params", bodyParamParams);
        try {
            HttpPost httpPost = new HttpPost(url);
            StringEntity stringEntity = new StringEntity(bodyParam.toJSONString(), StandardCharsets.UTF_8);
            httpPost.setHeader("Content-Type", "application/json;charset=UTF-8");
            httpPost.setEntity(stringEntity);
            CloseableHttpResponse response = client.execute(httpPost);
            StatusLine statusLine = response.getStatusLine();
            if (statusLine.getStatusCode() == 200) {
                HttpEntity responseEntity = response.getEntity();
                if (Objects.nonNull(responseEntity)) {
                    String responseBody = EntityUtils.toString(responseEntity, "UTF-8");
                    JSONObject jsonObject = JSONObject.parseObject(responseBody);
                    if (jsonObject.getInteger("code") == 0) {
                        JSONArray data = jsonObject.getJSONArray("data");
                        result.addAll(JSONArray.parseArray(data.toJSONString(), ProduceContentCrawlerVO.class));
                    }
                }
            }
        } catch (Exception e) {
            log.error("getCrawlerContentByPlan error", e);
        }
        log.info("getCrawlerContentByPlan耗时：{}", System.currentTimeMillis() - start);
        return result;
    }

}
