package com.tzld.longarticle.recommend.server.remote;

import com.alibaba.fastjson.JSONObject;
import com.tzld.longarticle.recommend.server.model.dto.RequestResult;
import lombok.extern.slf4j.Slf4j;
import okhttp3.*;
import org.springframework.stereotype.Service;

import javax.annotation.PostConstruct;
import java.util.Objects;
import java.util.concurrent.TimeUnit;

@Service
@Slf4j
public class WxArticleDeleteService {

    private OkHttpClient client;

    @PostConstruct
    public void init() {
        client = new OkHttpClient().newBuilder()
                .connectTimeout(15, TimeUnit.MINUTES)
                .readTimeout(15, TimeUnit.MINUTES)
                .writeTimeout(15, TimeUnit.MINUTES)
                .build();

    }

    public RequestResult<String> deleteArticle(String token, String msgId, int index) {
        RequestResult<String> result = new RequestResult<>();
        JSONObject jsonBody = new JSONObject();
        jsonBody.put("msg_id", msgId);
        jsonBody.put("article_idx", index);
        try {
            MediaType mediaType = MediaType.parse("application/json");
            RequestBody body = RequestBody.create(mediaType, JSONObject.toJSONString(jsonBody));
            Request request = new Request.Builder()
                    .url("https://api.weixin.qq.com/cgi-bin/message/mass/delete?access_token=" + token)
                    .method("POST", body)
                    .addHeader("Content-Type", "application/json")
                    .addHeader("Authorization", "Bearer sk-5DqYCa88kche6nwIWjLE1p4oMm8nXrR9kQMKbBolNAWERu7q")
                    .build();
            Response response = client.newCall(request).execute();

            String responseContent = response.body().string();
            result.setResponseStr(responseContent);
            log.info("wx delete api responseContent = {}", responseContent);
            if (response.isSuccessful()) {
                JSONObject obj = JSONObject.parseObject(responseContent);
                if (Objects.nonNull(obj) && obj.containsKey("errcode")) {
                    if (obj.getInteger("errcode") == 0) {
                        result.setSuccess(true);
                        result.setResponse(obj.toJSONString());
                    } else {
                        result.setFailReason(obj.getString("errmsg"));
                    }
                } else {
                    result.setFailReason("response empty");
                }
            } else {
                JSONObject json = JSONObject.parseObject(responseContent);
                result.setFailReason("request error code:" + response.code() + " message:" + json.getString("error"));
            }
        } catch (Exception e) {
            log.error("wx delete api error", e);
            result.setSuccess(false);
            result.setFailReason("request error" + e.getMessage());
        }
        return result;
    }

    public RequestResult<String> clearQuota(String token, String appid) {
        RequestResult<String> result = new RequestResult<>();
        JSONObject jsonBody = new JSONObject();
        jsonBody.put("appid", appid);
        try {
            MediaType mediaType = MediaType.parse("application/json");
            RequestBody body = RequestBody.create(mediaType, JSONObject.toJSONString(jsonBody));
            Request request = new Request.Builder()
                    .url("https://api.weixin.qq.com/cgi-bin/clear_quota?access_token=" + token)
                    .method("POST", body)
                    .addHeader("Content-Type", "application/json")
                    .addHeader("Authorization", "Bearer sk-5DqYCa88kche6nwIWjLE1p4oMm8nXrR9kQMKbBolNAWERu7q")
                    .build();
            Response response = client.newCall(request).execute();

            String responseContent = response.body().string();
            result.setResponseStr(responseContent);
            log.info("wx clearQuota api responseContent = {}", responseContent);
            if (response.isSuccessful()) {
                JSONObject obj = JSONObject.parseObject(responseContent);
                if (Objects.nonNull(obj) && obj.containsKey("errcode")) {
                    if (obj.getInteger("errcode") == 0) {
                        result.setSuccess(true);
                        result.setResponse(obj.toJSONString());
                    } else {
                        result.setFailReason(obj.getString("errmsg"));
                    }
                } else {
                    result.setFailReason("response empty");
                }
            } else {
                JSONObject json = JSONObject.parseObject(responseContent);
                result.setFailReason("request error code:" + response.code() + " message:" + json.getString("error"));
            }
        } catch (Exception e) {
            log.error("wx clearQuota api error", e);
            result.setSuccess(false);
            result.setFailReason("request error" + e.getMessage());
        }
        return result;
    }
}
