package com.tzld.longarticle.recommend.server.remote.aigc;

import cn.hutool.core.io.resource.ResourceUtil;
import com.alibaba.fastjson.JSONObject;
import com.tzld.longarticle.recommend.server.common.HttpPoolFactory;
import com.tzld.longarticle.recommend.server.model.vo.IdNameVO;
import lombok.extern.slf4j.Slf4j;
import org.apache.http.HttpEntity;
import org.apache.http.StatusLine;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.entity.StringEntity;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.util.EntityUtils;
import org.springframework.stereotype.Service;

import java.nio.charset.StandardCharsets;
import java.util.List;
import java.util.Objects;

@Service
@Slf4j
public class AIGCCrawlerPlanSaveService {

    private final CloseableHttpClient client = HttpPoolFactory.aigcPool();

    public IdNameVO<String> createArticleUrlPlan(String planName, List<String> contentList, String tag, Integer crawlerMode) {
        IdNameVO<String> result = new IdNameVO<>();
        if (contentList == null || contentList.isEmpty()) {
            return null;
        }
        // 从配置中读取模板文件路径
        String templateFile = ResourceUtil.readUtf8Str("file/长文_抓取计划_根据url列表抓取.json");
        // 读取模板文件内容
        JSONObject template = JSONObject.parseObject(templateFile);
        String apiUrl = template.getString("api_url");
        JSONObject data = template.getJSONObject("data");
        // 设置请求参数
        JSONObject params = data.getJSONObject("params");
        params.put("name", planName);
        params.put("crawlerMode", crawlerMode);
        if (tag != null) {
            params.put("planTag", tag);
        }
        params.put("inputModeValues", contentList);
        // 将数据转为 JSON 格式
        String requestData = data.toString();
        try {
            HttpPost httpPost = new HttpPost(apiUrl);
            StringEntity stringEntity = new StringEntity(requestData, StandardCharsets.UTF_8);
            httpPost.setHeader("Content-Type", "application/json;charset=UTF-8");
            httpPost.setEntity(stringEntity);
            CloseableHttpResponse response = client.execute(httpPost);
            StatusLine statusLine = response.getStatusLine();
            if (statusLine.getStatusCode() == 200) {
                HttpEntity responseEntity = response.getEntity();
                if (Objects.nonNull(responseEntity)) {
                    String responseBody = EntityUtils.toString(responseEntity, "UTF-8");
                    JSONObject jsonObject = JSONObject.parseObject(responseBody);
                    if (jsonObject.getInteger("code") == 0) {
                        JSONObject responseData = jsonObject.getJSONObject("data");
                        String planId = responseData.getString("id");
                        String returnedPlanName = responseData.getString("name");
                        log.info("plan_name: {}, plan_id: {}", returnedPlanName, planId);
                        result.setId(planId);
                        result.setName(returnedPlanName);
                    }
                }
            }
        } catch (Exception e) {
            log.error("createArticleUrlPlan error", e);
        }
        return result;
    }
}
