package com.tzld.longarticle.recommend.server.remote.pq;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.tzld.longarticle.recommend.server.common.HttpPoolFactory;
import com.tzld.longarticle.recommend.server.model.cgi.PQVideoAuditResult;
import com.tzld.longarticle.recommend.server.model.param.PQResponse;
import com.tzld.longarticle.recommend.server.repository.crawler.ArticleRepository;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.http.HttpEntity;
import org.apache.http.StatusLine;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.entity.StringEntity;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.util.EntityUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.io.IOException;
import java.nio.charset.StandardCharsets;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;

@Service
@Slf4j
public class PQVideoAuditResultService {

    @Autowired
    ArticleRepository articleRepository;

    @Value("${pq.host:}")
    private String host;

    private final CloseableHttpClient client = HttpPoolFactory.aigcPool();
    private static final String url = "/longvideoapi/openapi/audit/queryTaskResult";

    public List<PQVideoAuditResult> getResult(List<Long> taskIds) {
        int retryTimes = 3;
        while (retryTimes > 0) {
            List<PQVideoAuditResult> result = post(taskIds);
            if (CollectionUtils.isNotEmpty(result)) {
                return result;
            }
            retryTimes--;
        }
        return new ArrayList<>();
    }

    public List<PQVideoAuditResult> post(List<Long> taskIds) {
        JSONObject params = new JSONObject();
        params.put("taskIds", taskIds);
        try {
            HttpPost httpPost = new HttpPost(host + url);
            StringEntity stringEntity = new StringEntity(params.toJSONString(), StandardCharsets.UTF_8);
            httpPost.setHeader("Content-Type", "application/json;charset=UTF-8");
            httpPost.setEntity(stringEntity);
            CloseableHttpResponse response = client.execute(httpPost);
            StatusLine statusLine = response.getStatusLine();
            if (statusLine.getStatusCode() == 200) {
                HttpEntity responseEntity = response.getEntity();
                if (Objects.nonNull(responseEntity)) {
                    String responseBody = EntityUtils.toString(responseEntity, "UTF-8");
                    PQResponse pqResponse = JSONObject.parseObject(responseBody, PQResponse.class);
                    if (pqResponse.getCode() == 0 && Objects.nonNull(pqResponse.getData())) {
                        return JSONArray.parseArray(JSONObject.toJSONString(pqResponse.getData()), PQVideoAuditResult.class);
                    } else {
                        return null;
                    }
                }
            }
        } catch (IOException e) {
            log.error("PQVideoAuditResultService error", e);
        }
        return null;
    }


}
