package com.tzld.longarticle.recommend.server.service;

import com.tzld.longarticle.recommend.server.mapper.crawler.ArticleContentLinkMapper;
import com.tzld.longarticle.recommend.server.model.vo.WxContentDetailResponse;
import com.tzld.longarticle.recommend.server.remote.WxFetchRemoteService;
import com.tzld.longarticle.recommend.server.repository.model.ArticleContentLink;
import com.tzld.longarticle.recommend.server.repository.model.ArticleContentLinkExample;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang.math.NumberUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.time.LocalDate;
import java.time.ZoneId;
import java.time.ZonedDateTime;
import java.util.List;

/**
 * @author dyp
 */
@Service
@Slf4j
public class JobService {

    @Autowired
    private ArticleContentLinkMapper articleContentLinkMapper;
    @Autowired
    private WxFetchRemoteService wxFetchRemoteService;

    private long seconds1d = 60 * 60 * 24 * 1000;
    private long seconds7d = 7 * seconds1d;

    public void syncViewCount() {
        ArticleContentLinkExample example = new ArticleContentLinkExample();


        LocalDate today = LocalDate.now();
        // 计算7天前的日期
        LocalDate dateSevenDaysAgo = today.minusDays(7);
        // 将日期转换为当天开始的时间（00:00:00）并指定时区
        ZonedDateTime zonedDateTime = dateSevenDaysAgo.atStartOfDay(ZoneId.systemDefault());
        // 将ZonedDateTime转换为时间戳（毫秒）
        long time = zonedDateTime.toInstant().toEpochMilli();


        example.createCriteria().andCreateTimeGreaterThanOrEqualTo(time);
        List<ArticleContentLink> links = articleContentLinkMapper.selectByExample(example);
        for (ArticleContentLink link : links) {
            WxContentDetailResponse content = wxFetchRemoteService.getContent(link.getContentLink());
            link.setTitle(content.getTitle());
            link.setViewCount(NumberUtils.toLong(String.valueOf(content.getViewCount()), 0L));
            link.setUpdateTime(System.currentTimeMillis());

            articleContentLinkMapper.updateByPrimaryKey(link);
        }


    }
}
