package com.tzld.longarticle.recommend.server.service;

import com.tzld.longarticle.recommend.server.mapper.crawler.ArticleContentLinkMapper;
import com.tzld.longarticle.recommend.server.mapper.crawler.ArticleGzhDeveloperMapper;
import com.tzld.longarticle.recommend.server.repository.model.ArticleContentLink;
import com.tzld.longarticle.recommend.server.repository.model.ArticleGzhDeveloper;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang.math.NumberUtils;
import org.apache.commons.lang3.StringUtils;
import org.dom4j.Document;
import org.dom4j.Element;
import org.dom4j.io.SAXReader;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.io.StringReader;
import java.util.ArrayList;
import java.util.List;

/**
 * @author dyp
 */
@Slf4j
@Service
public class MessageSendCallbackService {

    @Autowired
    private ArticleContentLinkMapper articleContentLinkMapper;

    @Autowired
    private ArticleGzhDeveloperMapper articleGzhDeveloperMapper;

    /**
     * https://developers.weixin.qq.com/doc/offiaccount/Message_Management/Batch_Sends_and_Originality_Checks.html#_8%E3%80%81%E4%BA%8B%E4%BB%B6%E6%8E%A8%E9%80%81%E7%BE%A4%E5%8F%91%E7%BB%93%E6%9E%9C
     */
    public void listenWx(String xmlData) {
        try {
            SAXReader saxReader = new SAXReader();
            Document document = saxReader.read(new StringReader(xmlData));
            Element root = document.getRootElement();

            String msgType = root.elementText("MsgType");
            String event = root.elementText("Event");
            if (!StringUtils.equals("event", msgType)
                    || !StringUtils.equals("MASSSENDJOBFINISH", event)) {
                return;
            }


            String gzhId = root.elementText("ToUserName");
            ArticleGzhDeveloper agd = articleGzhDeveloperMapper.selectByPrimaryKey(gzhId);
            if (agd == null) {
                // 不再体系内的账号，不记录链接
                return;
            }


            long createTime = NumberUtils.toLong(root.elementText("CreateTime"), 0) * 1000;

            Element articleUrlResultEle = root.element("ArticleUrlResult");
            Element resultListEle = articleUrlResultEle.element("ResultList");
            List<Element> itemEle = resultListEle.elements("item");
            List<ArticleContentLink> links = new ArrayList<>();
            for (Element item : itemEle) {
                ArticleContentLink link = new ArticleContentLink();
                link.setGzhId(gzhId);
                link.setIdx(NumberUtils.toInt(item.elementText("ArticleIdx"), 0));
                link.setContentLink(item.elementText("ArticleUrl"));
                link.setCreateTime(createTime);
                link.setTitle("");
                link.setUpdateTime(createTime);
                link.setViewCount(-1L);
                links.add(link);
            }
            articleContentLinkMapper.insertBatch(links);
        } catch (Exception e) {
            log.error("listenWx handle error {} ", xmlData, e);
        }
    }

}
