package com.tzld.longarticle.recommend.server.service.exterior.impl;

import com.alibaba.fastjson.JSON;
import com.aliyun.odps.data.Record;
import com.ctrip.framework.apollo.spring.annotation.ApolloJsonValue;
import com.tzld.longarticle.recommend.server.common.constant.TimeConstant;
import com.tzld.longarticle.recommend.server.common.enums.GhTypeEnum;
import com.tzld.longarticle.recommend.server.common.enums.SecretEnum;
import com.tzld.longarticle.recommend.server.common.enums.StatusEnum;
import com.tzld.longarticle.recommend.server.common.enums.cgi.ReplyStrategyServiceEnum;
import com.tzld.longarticle.recommend.server.common.response.CommonResponse;
import com.tzld.longarticle.recommend.server.common.response.ExceptionCodeEnum;
import com.tzld.longarticle.recommend.server.mapper.growth.GhDetailMapper;
import com.tzld.longarticle.recommend.server.model.bo.MiniData;
import com.tzld.longarticle.recommend.server.model.cgi.BucketDataParam;
import com.tzld.longarticle.recommend.server.model.cgi.GroupData;
import com.tzld.longarticle.recommend.server.model.cgi.MsgData;
import com.tzld.longarticle.recommend.server.model.cgi.ReplyBucketData;
import com.tzld.longarticle.recommend.server.model.param.PushMessageParam;
import com.tzld.longarticle.recommend.server.model.vo.PushMessageVo;
import com.tzld.longarticle.recommend.server.model.vo.ReportUvVo;
import com.tzld.longarticle.recommend.server.repository.model.GhDetail;
import com.tzld.longarticle.recommend.server.repository.model.GhDetailExample;
import com.tzld.longarticle.recommend.server.service.exterior.AccessTokenService;
import com.tzld.longarticle.recommend.server.service.exterior.ThirdPartyService;
import com.tzld.longarticle.recommend.server.service.strategy.reply.ReplyStrategyService;
import com.tzld.longarticle.recommend.server.util.DateUtils;
import com.tzld.longarticle.recommend.server.util.LarkRobotUtil;
import lombok.Data;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationContext;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import com.tzld.longarticle.recommend.server.remote.ODPSManager;

import javax.annotation.PostConstruct;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Objects;

@Slf4j
@Service
public class ThirdPartyServiceImpl implements ThirdPartyService {

    @Autowired
    private ApplicationContext applicationContext;

    @Autowired
    private AccessTokenService accessTokenService;

    @Autowired
    private ODPSManager odpsManager;

    @Autowired
    private GhDetailMapper ghDetailMapper;

    @ApolloJsonValue("${canViewReportDate:2024-10-31}")
    private String canViewReportDate;

    private Map<String, ReplyStrategyService> strategyServiceMap;

    @PostConstruct
    public void init() {
        strategyServiceMap = applicationContext.getBeansOfType(ReplyStrategyService.class);
    }


    @Override
    public CommonResponse<List<PushMessageVo>> getPushMessage(PushMessageParam param) {
        if (param == null || StringUtils.isEmpty(param.getGhId()) || StringUtils.isEmpty(param.getAccessToken())) {
            return CommonResponse.create(ExceptionCodeEnum.PARAM_ERROR, "参数错误");
        }
        if (!accessTokenService.validateAccessToken(param.getAccessToken())) {
            return CommonResponse.create(ExceptionCodeEnum.PARAM_ERROR, "accessToken错误或者已失效");
        }
        SecretEnum secretEnum = accessTokenService.getSecretEnum(param.getAccessToken());
        if (secretEnum == null) {
            return CommonResponse.create(ExceptionCodeEnum.PARAM_ERROR, "获取secret失败");
        }
        log.info("getPushMessage param={} secretEnum desc={}", param, secretEnum.desc);
        GhDetailExample example = new GhDetailExample();
        example.createCriteria().andTypeEqualTo(GhTypeEnum.THIRD_PARTY_GH.type).andGhIdEqualTo(param.getGhId());
        List<GhDetail> ghDetails = ghDetailMapper.selectByExample(example);
        if (CollectionUtils.isEmpty(ghDetails)) {
            return CommonResponse.create(404, "ghId不存在，请联系管理员配置");
        }
        String channel = ghDetails.get(0).getChannel();
        if (channel == null) {
            LarkRobotUtil.sendMessage("channel不存在，请查看详情 ghId=", param.getGhId());
            return CommonResponse.create(404, "ghId异常，请联系管理员检查");
        }
        if (!Objects.equals(secretEnum.channel, channel)) {
            LarkRobotUtil.sendMessage(String.format("channel异常 secretEnum.channel=%s ghDetail.channel=%s ghId=%s",
                    secretEnum.channel, channel, param.getGhId()));
            return CommonResponse.create(404, "ghId异常，请联系管理员检查");
        }
        List<PushMessageVo> pushMessageVoList = new ArrayList<>();
        ReplyBucketData replyBucketData = getPushMessageData(param, channel);
        log.info("replyBucketData={}", JSON.toJSONString(replyBucketData));
        if (replyBucketData == null) {
            log.error("获取推送策略数据失败");
            return CommonResponse.create(500, "系统异常，获取失败");
        }
        List<GroupData> groupList = replyBucketData.getGroupList();
        for (GroupData groupData : groupList) {
            if (CollectionUtils.isEmpty(groupData.getMsgDataList())) {
                continue;
            }
            PushMessageVo pushMessageVo = new PushMessageVo();
            List<MiniData> components = new ArrayList<>();
            for (MsgData msgData : groupData.getMsgDataList()) {
                MiniData miniData = new MiniData();
                BeanUtils.copyProperties(msgData, miniData);
                components.add(miniData);
            }
            pushMessageVo.setGroupIdx(groupData.getGroupIndex());
            pushMessageVo.setComponents(components);
            pushMessageVoList.add(pushMessageVo);
        }
        return CommonResponse.success(pushMessageVoList);
    }

    @Override
    public CommonResponse<List<ReportUvVo>> getReportUv(String date, String accessToken) {
        if (StringUtils.isEmpty(date) || !DateUtils.isValidDate(date) || StringUtils.isEmpty(accessToken)) {
            return CommonResponse.create(ExceptionCodeEnum.PARAM_ERROR, "参数错误");
        }
        if (!accessTokenService.validateAccessToken(accessToken)) {
            return CommonResponse.create(ExceptionCodeEnum.PARAM_ERROR, "accessToken错误或者已失效");
        }
        SecretEnum secretEnum = accessTokenService.getSecretEnum(accessToken);
        if (secretEnum == null) {
            return CommonResponse.create(ExceptionCodeEnum.PARAM_ERROR, "获取secret失败");
        }
        if (!DateUtils.isValidDate(canViewReportDate)) {
            return CommonResponse.create(ExceptionCodeEnum.PARAM_ERROR, "系统异常");
        }
        //llf只返回2024-11-30后的数据
        if (secretEnum == SecretEnum.SECRET_ENUM_2) {
            long targetTime = DateUtils.dateStrToTimestamp(date, "yyyy-MM-dd");
            long limitTime = DateUtils.dateStrToTimestamp("2024-11-30", "yyyy-MM-dd");
            if (targetTime < limitTime) {
                return CommonResponse.create(500, "数据不存在");
            }
        }

        //10点后可查询前一天数据
        long nowTimestamp = System.currentTimeMillis() / 1000;
        long limitTime = nowTimestamp - 34L * TimeConstant.HOUR;
        long targetTime = DateUtils.dateStrToTimestamp(date, "yyyy-MM-dd");
        String channel = secretEnum.channel;
        if (targetTime > limitTime) {
            return CommonResponse.create(500, "数据不存在");
        }
        String dt = date.replace("-", "").substring(0, 8);
        String sql = String.format("SELECT * FROM alg_growth_3rd_gh_reply_uv_report WHERE dt = %s AND channel = '%s';",
                dt, channel);
        List<ReportUvVo> res = new ArrayList<>();
        List<Record> recordList = odpsManager.query(sql);
        if (CollectionUtils.isEmpty(recordList)) {
            return CommonResponse.success(res);
        }
        for (Record record : recordList) {
            ReportUvVo reportUvVo = new ReportUvVo();
            reportUvVo.setGhId(record.getString(0));
            reportUvVo.setUv(record.getBigint(1));
            res.add(reportUvVo);
        }
        return CommonResponse.success(res);
    }

    private ReplyBucketData getPushMessageData(PushMessageParam param, String channel) {
        log.info("strategyServiceMap={}", JSON.toJSONString(strategyServiceMap));
        for (Map.Entry<String, ReplyStrategyService> stringReplyStrategyServiceEntry : strategyServiceMap.entrySet()) {
            ReplyStrategyService replyStrategyService = stringReplyStrategyServiceEntry.getValue();
            // 使用策略层
            if (replyStrategyService.support(ReplyStrategyServiceEnum.PUSH_MESSAGE_STRATEGY_V1)) {
                BucketDataParam bucketDataParam = new BucketDataParam();
                bucketDataParam.setGhId(param.getGhId());
                bucketDataParam.setChannel(channel);
                return replyStrategyService.getResult(bucketDataParam);
            }
        }
        // 无执行策略 不会走到这里
        return null;
    }
}
