package com.tzld.longarticle.recommend.server.service.exterior.impl;

import com.alibaba.fastjson.JSON;
import com.tzld.longarticle.recommend.server.common.enums.cgi.ReplyStrategyServiceEnum;
import com.tzld.longarticle.recommend.server.common.response.CommonResponse;
import com.tzld.longarticle.recommend.server.common.response.ExceptionCodeEnum;
import com.tzld.longarticle.recommend.server.mapper.growth.ReplyStaffMapper;
import com.tzld.longarticle.recommend.server.model.cgi.BucketDataParam;
import com.tzld.longarticle.recommend.server.model.cgi.GroupData;
import com.tzld.longarticle.recommend.server.model.cgi.MsgData;
import com.tzld.longarticle.recommend.server.model.cgi.ReplyBucketData;
import com.tzld.longarticle.recommend.server.model.param.WeComPushMessageParam;
import com.tzld.longarticle.recommend.server.model.vo.WeComPushMessageVo;
import com.tzld.longarticle.recommend.server.repository.model.ReplyStaff;
import com.tzld.longarticle.recommend.server.repository.model.ReplyStaffExample;
import com.tzld.longarticle.recommend.server.service.exterior.WeComService;
import com.tzld.longarticle.recommend.server.service.strategy.reply.ReplyStrategyService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationContext;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import javax.annotation.PostConstruct;
import java.nio.charset.StandardCharsets;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;


@Slf4j
@Service
public class WeComServiceImpl implements WeComService {

    private static final String CORP_ID = "wwa4015dc7d652a21f";

    private static final int MAX_BYTES = 64;

    @Autowired
    private ApplicationContext applicationContext;

    @Autowired
    private ReplyStaffMapper replyStaffMapper;

    private Map<String, ReplyStrategyService> strategyServiceMap;

    @PostConstruct
    public void init() {
        strategyServiceMap = applicationContext.getBeansOfType(ReplyStrategyService.class);
    }

    @Override
    public CommonResponse<List<WeComPushMessageVo>> getPushMessage(WeComPushMessageParam param) {
        if (param == null || StringUtils.isEmpty(param.getCorpId()) || !CORP_ID.equals(param.getCorpId()) || StringUtils.isEmpty(param.getUserId())) {
            return CommonResponse.create(ExceptionCodeEnum.PARAM_ERROR, "参数错误");
        }
        List<WeComPushMessageVo> res = new ArrayList<>();
        ReplyBucketData replyBucketData = getPushMessageData(param);
        List<GroupData> groupList = replyBucketData.getGroupList();
        log.info("groupList={}", groupList);
        if (CollectionUtils.isEmpty(groupList)) {
            return CommonResponse.create(500, "数据异常");
        }
        ReplyStaffExample example = new ReplyStaffExample();
        example.createCriteria().andUserIdEqualTo(param.getUserId()).andIsDeleteEqualTo(0);
        List<ReplyStaff> replyStaffs = replyStaffMapper.selectByExample(example);
        if (CollectionUtils.isEmpty(replyStaffs)) {
            return CommonResponse.create(ExceptionCodeEnum.PARAM_ERROR, "用户查询不到");
        }
        String name = replyStaffs.get(0).getName();
        for (GroupData groupData : groupList) {
            if (CollectionUtils.isEmpty(groupData.getMsgDataList())) {
                continue;
            }
            WeComPushMessageVo weComPushMessageVo = new WeComPushMessageVo();
            weComPushMessageVo.setGroupIndex(groupData.getGroupIndex());
            List<MsgData> msgDataList = groupData.getMsgDataList();
            for (MsgData msgData : msgDataList) {
                if (msgData.getTitle().getBytes(StandardCharsets.UTF_8).length > MAX_BYTES) {
                    String s = truncateString(msgData.getTitle(), MAX_BYTES - 3) + "...";
                    msgData.setTitle(s);
                }
            }
            MsgData msgData = new MsgData();
            msgData.setMsgType(3);
            msgData.setTitle(String.format("很高兴认识您！我是%s，每天给您推荐精彩视频~", name));
            msgDataList.add(0, msgData);
            weComPushMessageVo.setMsgDataList(msgDataList);
            res.add(weComPushMessageVo);
        }
        log.info("res={}", res);
        return CommonResponse.success(res);
    }

    private ReplyBucketData getPushMessageData(WeComPushMessageParam param) {
        log.info("strategyServiceMap={}", JSON.toJSONString(strategyServiceMap));
        for (Map.Entry<String, ReplyStrategyService> stringReplyStrategyServiceEntry : strategyServiceMap.entrySet()) {
            ReplyStrategyService replyStrategyService = stringReplyStrategyServiceEntry.getValue();
            // 使用策略层
            if (replyStrategyService.support(ReplyStrategyServiceEnum.WE_COM_PUSH_MESSAGE_STRATEGY_V1)) {
                BucketDataParam bucketDataParam = new BucketDataParam();
                bucketDataParam.setGhId(param.getUserId());
                return replyStrategyService.getResult(bucketDataParam);
            }
        }
        // 无执行策略 不会走到这里
        return null;
    }

    private String truncateString(String input, int maxBytes) {
        if (input == null || maxBytes <= 0) {
            return "";
        }

        byte[] bytes = input.getBytes(StandardCharsets.UTF_8);
        if (bytes.length <= maxBytes) {
            return input; // 如果字节数已经在限制内，直接返回原字符串
        }

        // 截取字节数组
        byte[] truncatedBytes = new byte[maxBytes];
        System.arraycopy(bytes, 0, truncatedBytes, 0, maxBytes);

        // 将截取的字节数组转换回字符串
        String truncatedString = new String(truncatedBytes, StandardCharsets.UTF_8);

        // 处理可能的字符截断问题
        // 如果截取后字符串的字节数仍然大于 maxBytes，向前查找直到找到有效字符
        while (truncatedString.getBytes(StandardCharsets.UTF_8).length > maxBytes) {
            truncatedString = truncatedString.substring(0, truncatedString.length() - 1);
        }

        return truncatedString;
    }
}
