package com.tzld.longarticle.recommend.server.service.recommend;

import com.tzld.longarticle.recommend.server.common.enums.recommend.ContentPoolEnum;
import com.tzld.longarticle.recommend.server.mapper.longArticle.LongArticleBaseMapper;
import com.tzld.longarticle.recommend.server.model.param.PublishContentFilterContentItem;
import com.tzld.longarticle.recommend.server.model.param.PublishContentFilterParam;
import com.tzld.longarticle.recommend.server.model.vo.PublishContentFilterResultVO;
import com.tzld.longarticle.recommend.server.util.TitleSimilarCheckUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections4.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import java.util.ArrayList;
import java.util.List;
import java.util.Set;

@Service
@Slf4j
public class PublishContentFilterService {

    @Autowired
    private LongArticleBaseMapper longArticleBaseMapper;

    public List<PublishContentFilterResultVO> publishContentFilter(PublishContentFilterParam param) {
        List<PublishContentFilterResultVO> result = new ArrayList<>();
        if (CollectionUtils.isEmpty(param.getContentList())) {
            return result;
        }
        List<String> filterTitleList = longArticleBaseMapper.getFilterColdLongArticleTitle();
        List<Set<Character>> firstSecondTitleCache = TitleSimilarCheckUtil.makeCache(filterTitleList);
        for (PublishContentFilterContentItem contentItem : param.getContentList()) {
            if (!StringUtils.hasText(contentItem.getTitle())) {
                continue;
            }
            boolean isDuplicate = false;
            if (ContentPoolEnum.autoArticlePoolLevel4.getContentPool().equals(contentItem.getPoolLevel())) {
                isDuplicate = TitleSimilarCheckUtil.isDuplicateContentByCache(contentItem.getTitle(),
                        firstSecondTitleCache, TitleSimilarCheckUtil.SIMILARITY_THRESHOLD);
            }
            if (isDuplicate) {
                PublishContentFilterResultVO vo = new PublishContentFilterResultVO();
                vo.setSourceId(contentItem.getSourceId());
                vo.setFilterReason("退场机制过滤");
                result.add(vo);
            }
        }
        return result;
    }
}
