package com.tzld.longarticle.recommend.server.service.recommend.filter.strategy;

import com.tzld.longarticle.recommend.server.common.enums.recommend.ArticleTypeEnum;
import com.tzld.longarticle.recommend.server.model.dto.Content;
import com.tzld.longarticle.recommend.server.remote.ArticleListRemoteService;
import com.tzld.longarticle.recommend.server.model.entity.crawler.Article;
import com.tzld.longarticle.recommend.server.service.recommend.filter.FilterParam;
import com.tzld.longarticle.recommend.server.service.recommend.filter.FilterResult;
import com.tzld.longarticle.recommend.server.service.recommend.filter.FilterStrategy;
import com.tzld.longarticle.recommend.server.util.TitleSimilarCheckUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @author dyp
 */
@Component
@Slf4j
public class HistoryTitleForFwhColdStartStrategy implements FilterStrategy {

    @Autowired
    private ArticleListRemoteService articleListRemoteService;

    private static final List<Integer> allIndex = Arrays.asList(1, 2, 3, 4, 5, 6, 7, 8);


    @Override
    public FilterResult filter(FilterParam param) {
        long start = System.currentTimeMillis();
        FilterResult filterResult = new FilterResult();
        List<String> result = new ArrayList<>();
        List<Content> filterContents = new ArrayList<>();
        List<Article> allArticleList = articleListRemoteService.articleList(param.getGhId(), allIndex, ArticleTypeEnum.QUNFA.getVal());
        List<String> allTitleList = allArticleList.stream().map(Article::getTitle).distinct().collect(Collectors.toList());

        for (Content content : param.getContents()) {
            boolean isDuplicate = TitleSimilarCheckUtil.isDuplicateContent(content.getTitle(), allTitleList, TitleSimilarCheckUtil.SIMILARITY_THRESHOLD);
            if (!isDuplicate) {
                result.add(content.getId());
            } else {
                content.setFilterReason("历史已发布文章");
                filterContents.add(content);
            }
        }
        filterResult.setContentIds(result);
        filterResult.setFilterContent(filterContents);
        return filterResult;
    }

}
