package com.tzld.longarticle.recommend.server.service.recommend.filter.strategy;

import com.tzld.longarticle.recommend.server.common.ThreadPoolFactory;
import com.tzld.longarticle.recommend.server.model.dto.Content;
import com.tzld.longarticle.recommend.server.remote.ArticleListRemoteService;
import com.tzld.longarticle.recommend.server.model.entity.crawler.Article;
import com.tzld.longarticle.recommend.server.service.recommend.config.AccountContentPoolConfigService;
import com.tzld.longarticle.recommend.server.service.recommend.filter.FilterParam;
import com.tzld.longarticle.recommend.server.service.recommend.filter.FilterResult;
import com.tzld.longarticle.recommend.server.service.recommend.filter.FilterStrategy;
import com.tzld.longarticle.recommend.server.util.TitleSimilarCheckUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections4.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.util.StringUtils;

import java.util.*;
import java.util.concurrent.CountDownLatch;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Future;
import java.util.stream.Collectors;

/**
 * @author dyp
 */
@Component
@Slf4j
public class HistoryTitleStrategy implements FilterStrategy {

    @Autowired
    private ArticleListRemoteService articleListRemoteService;
    @Autowired
    private AccountContentPoolConfigService accountContentPoolConfigService;

    private static final List<Integer> firstSecondIndex = Arrays.asList(1, 2);
    private static final List<Integer> allIndex = Arrays.asList(1, 2, 3, 4, 5, 6, 7, 8);

    private final ExecutorService pool = ThreadPoolFactory.deDuplicatePool();

    @Override
    public FilterResult filter(FilterParam param) {
        FilterResult filterResult = new FilterResult();
        List<String> result = new ArrayList<>(param.getContents().size());
        List<Content> filterContents = new ArrayList<>();
        List<Article> allArticleList = articleListRemoteService.articleList(param.getGhId(), allIndex, param.getType());
        List<String> allTitleList = allArticleList.stream().map(Article::getTitle).distinct().collect(Collectors.toList());
        List<String> firstSecondTitleList = allArticleList.stream()
                .filter(article -> firstSecondIndex.contains(article.getItemIndex()))
                .map(Article::getTitle).distinct().collect(Collectors.toList());

        List<String> firstSecondContentPool = new ArrayList<>();
        String[] contentPoolConfig = accountContentPoolConfigService.getContentPools(param.getAccountName());
        if (Objects.nonNull(contentPoolConfig)) {
            firstSecondContentPool.addAll(Arrays.asList(contentPoolConfig[0], contentPoolConfig[1]));
        }
        List<Set<Character>> firstSecondTitleCache = TitleSimilarCheckUtil.makeCache(firstSecondTitleList);
        List<Set<Character>> allTitleCache = TitleSimilarCheckUtil.makeCache(allTitleList);
        // TODO: batching for parallelism
        for (Content content : param.getContents()) {
            boolean isDuplicate;
            if (CollectionUtils.isNotEmpty(firstSecondContentPool) && firstSecondContentPool.contains(content.getContentPoolType())) {
                // 四个内容池 配置 判断头条，次头条
                isDuplicate = TitleSimilarCheckUtil.isDuplicateContentByCache(content.getTitle(), firstSecondTitleCache, TitleSimilarCheckUtil.SIMILARITY_THRESHOLD);
            } else {
                isDuplicate = TitleSimilarCheckUtil.isDuplicateContentByCache(content.getTitle(), allTitleCache, TitleSimilarCheckUtil.SIMILARITY_THRESHOLD);
            }
            if (isDuplicate) {
                content.setFilterReason("历史已发布文章");
                filterContents.add(content);
            } else {
                result.add(content.getId());
            }
        }
        filterResult.setContentIds(result);
        filterResult.setFilterContent(filterContents);
        return filterResult;
    }

}
