package com.tzld.longarticle.recommend.server.service.recommend.filter.strategy;

import com.ctrip.framework.apollo.spring.annotation.ApolloJsonValue;
import com.tzld.longarticle.recommend.server.common.ThreadPoolFactory;
import com.tzld.longarticle.recommend.server.common.enums.StatusEnum;
import com.tzld.longarticle.recommend.server.model.dto.Content;
import com.tzld.longarticle.recommend.server.model.entity.crawler.ArticleSensitive;
import com.tzld.longarticle.recommend.server.model.entity.longArticle.ArticleUnsafeTitle;
import com.tzld.longarticle.recommend.server.remote.ArticleSensitiveRemoteService;
import com.tzld.longarticle.recommend.server.repository.crawler.ArticleSensitiveRepository;
import com.tzld.longarticle.recommend.server.repository.longArticle.ArticleUnsafeTitleRepository;
import com.tzld.longarticle.recommend.server.service.recommend.filter.FilterParam;
import com.tzld.longarticle.recommend.server.service.recommend.filter.FilterResult;
import com.tzld.longarticle.recommend.server.service.recommend.filter.FilterStrategy;
import com.tzld.longarticle.recommend.server.util.TitleSimilarCheckUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections4.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.concurrent.ExecutorService;

@Component
@Slf4j
public class SensitiveStrategy implements FilterStrategy {

    @Autowired
    private ArticleSensitiveRemoteService articleSensitiveRemoteService;
    @Autowired
    private ArticleSensitiveRepository articleSensitiveRepository;
    @Autowired
    private ArticleUnsafeTitleRepository articleUnsafeTitleRepository;

    @ApolloJsonValue("${UnSafeTitles:[]}")
    private static List<String> UnSafeTitles;

    private final ExecutorService pool = ThreadPoolFactory.deDuplicatePool();

    @Override
    public FilterResult filter(FilterParam param) {
        FilterResult filterResult = new FilterResult();
        List<String> result = new ArrayList<>(param.getContents().size());
        List<Content> filterContents = new ArrayList<>();

//        Map<String, String> titleMd5Map = new HashMap<>();
//        Map<String, ArticleSensitive> articleSensitiveMap = new HashMap<>();
//        List<String> md5List = new ArrayList<>();
//        for (Content content : param.getContents()) {
//            String md5 = Md5Util.encoderByMd5(content.getTitle());
//            md5List.add(md5);
//            titleMd5Map.put(content.getTitle(), md5);
//        }
//        List<ArticleSensitive> articleSensitiveList = getArticleSensitive(md5List);
//        if (CollectionUtil.isNotEmpty(articleSensitiveList)) {
//            articleSensitiveMap = articleSensitiveList.stream().collect(Collectors.toMap(ArticleSensitive::getMd5, o -> o));
//        }
        List<String> allUnSafeTitles = UnSafeTitles;
        List<ArticleUnsafeTitle> articleUnsafeTitleList = articleUnsafeTitleRepository.getByStatus(StatusEnum.ONE.getCode());
        if (CollectionUtils.isNotEmpty(articleUnsafeTitleList)) {
            for (ArticleUnsafeTitle articleUnsafeTitle : articleUnsafeTitleList) {
                allUnSafeTitles.add(articleUnsafeTitle.getTitle());
            }
        }
        List<Set<Character>> unsafeTitleCache = TitleSimilarCheckUtil.makeCache(allUnSafeTitles);
        for (Content content : param.getContents()) {
            try {
                if (TitleSimilarCheckUtil.isDuplicateContentByCache(content.getTitle(), unsafeTitleCache,
                        TitleSimilarCheckUtil.SIMILARITY_THRESHOLD)) {
                    content.setFilterReason("安全违规");
                    filterContents.add(content);
                } else {
                    result.add(content.getId());
                }
            } catch (Exception e) {
                log.error("similar check error ", e);
            }
        }
        filterResult.setContentIds(result);
        filterResult.setFilterContent(filterContents);
        return filterResult;
    }

    public List<ArticleSensitive> getArticleSensitive(List<String> md5List) {
        return articleSensitiveRepository.findAllById(md5List);
    }

}
