package com.tzld.longarticle.recommend.server.service.recommend.rank.strategy;


import com.tzld.longarticle.recommend.server.common.enums.recommend.ScoreStrategyEnum;
import com.tzld.longarticle.recommend.server.model.dto.Content;
import com.tzld.longarticle.recommend.server.service.recommend.config.AccountContentPoolConfigService;
import com.tzld.longarticle.recommend.server.service.recommend.rank.RankItem;
import com.tzld.longarticle.recommend.server.service.recommend.rank.RankParam;
import com.tzld.longarticle.recommend.server.service.recommend.rank.RankResult;
import com.tzld.longarticle.recommend.server.service.recommend.rank.RankStrategy;
import com.tzld.longarticle.recommend.server.service.recommend.score.ScoreResult;
import com.tzld.longarticle.recommend.server.service.recommend.score.ScoreService;
import com.tzld.longarticle.recommend.server.util.CommonCollectionUtils;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Map;

/**
 * @author dyp
 */
@Service
@Slf4j
public class FwhColdStartRankStrategy implements RankStrategy {

    @Autowired
    private ScoreService scoreService;
    @Autowired
    private AccountContentPoolConfigService accountContentPoolConfigService;

    public RankResult rank(RankParam param) {

        ScoreResult scoreResult = scoreService.score(RankStrategy.convertToScoreParam(param));

        Map<String, Map<String, Double>> scoreMap = scoreResult.getScoreMap();
        String[] contentPools = accountContentPoolConfigService.getContentPools(param.getAccountName());

        List<RankItem> items = CommonCollectionUtils.toList(param.getContents(), c -> {
            RankItem item = new RankItem();
            c.setHisPublishArticleList(null);
            item.setContent(c);
            item.setScoreMap(scoreMap.get(c.getId()));
            double score;
            if (contentPools[0].equals(item.getContent().getContentPoolType())
                    || contentPools[1].equals(item.getContent().getContentPoolType())) {
                score = item.getScore(ScoreStrategyEnum.SIMILARITY.value())
                        + item.getScore(ScoreStrategyEnum.CATEGORY.value())
                        + item.getScore(ScoreStrategyEnum.FLOW_CTL_DECREASE.value());
                if (item.getScore(ScoreStrategyEnum.PUBLISH_TIMES.value()) >= 0) {
                    score += item.getScore(ScoreStrategyEnum.VIEW_COUNT_RATE.value());
                }
            } else {
                score = item.getScore(ScoreStrategyEnum.SIMILARITY.value())
                        + item.getScore(ScoreStrategyEnum.CATEGORY.value())
                        + item.getScore(ScoreStrategyEnum.ACCOUNT_PRE_DISTRIBUTE.value())
                        + item.getScore(ScoreStrategyEnum.PUBLISH_TIMES.value())
                        + item.getScore(ScoreStrategyEnum.CRAWLER_DAYS_DECREASE_STRATEGY.value())
                        + item.getScore(ScoreStrategyEnum.FLOW_CTL_DECREASE.value());
            }
            item.setScore(score);
            return item;
        });

        // 1 排序
        Collections.sort(items, (o1, o2) -> -Double.compare(o1.getScore(), o2.getScore()));
        // 2 相似去重
        List<Content> contents = CommonCollectionUtils.toList(items, RankItem::getContent);

        // 3 选文章
        List<Content> result = new ArrayList<>();
        int size = param.getSize() * param.getUserGroupIds().size();
        result.addAll(contents.subList(0, Math.min(contents.size(), size)));

        return new RankResult(result);
    }

}
