package com.tzld.longarticle.recommend.server.service.recommend.score.strategy;

import com.tzld.longarticle.recommend.server.service.recommend.score.Score;
import com.tzld.longarticle.recommend.server.service.recommend.score.ScoreParam;
import com.tzld.longarticle.recommend.server.service.recommend.score.ScoreStrategy;
import com.tzld.longarticle.recommend.server.util.CommonCollectionUtils;
import com.tzld.longarticle.recommend.server.util.NormalizationUtils;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

import java.util.Collections;
import java.util.List;

/**
 * @author dyp
 */
@Component
@Slf4j
public class ViewCountStrategy implements ScoreStrategy {

    @Override
    public List<Score> score(ScoreParam param) {
        long start = System.currentTimeMillis();

        if (CollectionUtils.isEmpty(param.getContents())) {
            return Collections.emptyList();
        }

        double min = param.getContents().stream()
                .map(c -> c.getCrawlerViewCount() == null ? 0.0 : c.getCrawlerViewCount())
                .min(Double::compareTo)
                .orElse(0.0);
        double max = param.getContents().stream()
                .map(c -> c.getCrawlerViewCount() == null ? 0.0 : c.getCrawlerViewCount())
                .max(Double::compareTo)
                .orElse(0.0);

        List<Score> scores = CommonCollectionUtils.toList(param.getContents(), c -> {
            Score score = new Score();
            score.setContentId(c.getId());
            double viewCount = c.getCrawlerViewCount() == null ? 0.0 : c.getCrawlerViewCount();
            score.setScore(NormalizationUtils.minMax(viewCount, min, max));
            score.setStrategy(this);
            return score;
        });
        return scores;
    }

}
