package com.tzld.longarticle.recommend.server.util.feishu;

import com.alibaba.fastjson.JSONObject;
import com.tzld.longarticle.recommend.server.util.feishu.model.FeishuChatListResult;
import com.tzld.longarticle.recommend.server.util.feishu.model.FeishuGroup;
import com.tzld.longarticle.recommend.server.util.http.HttpClientUtils;
import com.tzld.longarticle.recommend.server.util.http.HttpResponseContent;
import org.slf4j.LoggerFactory;
import org.springframework.data.util.Pair;

import java.util.HashMap;

public class FeiShu {

    private static final org.slf4j.Logger LOGGER = LoggerFactory.getLogger(FeiShu.class);

    public static final String URL_FEISHU_ACCESSTOKEN = "https://open.feishu.cn/open-apis/auth/v3/tenant_access_token/internal";
    public static final String URL_FEISHU_CHAT_LIST = "https://open.feishu.cn/open-apis/chat/v4/list?page_size=200";
    public static final String URL_FEISHU_SEND_MESSAGE = "https://open.feishu.cn/open-apis/message/v4/send";
    public static final String URL_FEISHU_GROUP_MEMBERS = "https://open.feishu.cn/open-apis/im/v1/chats/:chat_id/members?member_id_type=open_id";
    public static final String URL_FEISHU_REPLY_MESSAGE = "https://open.feishu.cn/open-apis/im/v1/messages/:message_id/reply";
    public static final String URL_FEISHU_CARD_MESSAGE_UPDATE = "https://open.feishu.cn/open-apis/im/v1/messages/:message_id";
    public static final String APPID = "cli_a22acf2916b8500e";
    public static final String APPSECRET = "tE0xAB2gZTMlBGdPczCGLcmpRlZQm5CQ";

    public static Pair<String, Integer> requestAccessToken() {
        Pair<String, Integer> result = Pair.of("", 0);
        long startTime = System.currentTimeMillis();
        int retryCount = 0;

        while (retryCount < 3) {
            try {
                HashMap<String, String> params = new HashMap<>();
                params.put("app_id", APPID);
                params.put("app_secret", APPSECRET);
                HttpResponseContent hrc = HttpClientUtils.postForm(URL_FEISHU_ACCESSTOKEN, params, new HashMap<>());
                LOGGER.info("feishu accessToken response = {}", hrc.getBodyContent());
                JSONObject response = JSONObject.parseObject(hrc.getBodyContent());
                String accessToken = response.getString("tenant_access_token");
                int expireSeconds = response.getIntValue("expire");
                return Pair.of(accessToken, expireSeconds);
            } catch (Exception e) {
                LOGGER.error("requestFeishuAccessToken, error ，retryCount = {}, cost = {}", retryCount, System.currentTimeMillis() - startTime, e);
            }
            retryCount++;
        }

        return result;

    }

    public static void main(String[] args) {
        HashMap<String, String> params = new HashMap<>();
        params.put("app_id", APPID);
        params.put("app_secret", APPSECRET);
        HttpResponseContent hrc = HttpClientUtils.postForm(URL_FEISHU_ACCESSTOKEN, params, new HashMap<>());
        System.out.println(hrc.getBodyContent());
        JSONObject response = JSONObject.parseObject(hrc.getBodyContent());
        String accessToken = response.getString("tenant_access_token");

        HashMap<String, String> header = new HashMap<>();
        header.put("Authorization", "Bearer " + accessToken);
        header.put("content-type", "application/json; charset=utf-8");

        //获取群聊ID
        HttpResponseContent chatHrc = HttpClientUtils.get(URL_FEISHU_CHAT_LIST, header);
        JSONObject responseJSON = JSONObject.parseObject(chatHrc.getBodyContent());
        String chatData = responseJSON.getString("data");
        System.out.println(chatData);
        System.out.println("-------------------");
        FeishuChatListResult feishuChatListResult = JSONObject.parseObject(chatData, FeishuChatListResult.class);
        for (FeishuGroup group : feishuChatListResult.getGroups()) {
            if ("重要！Ai Dit公众号推送失败告警群".equals(group.getName())) {
                System.out.println(group.getChat_id());
            }
        }
    }
}
