package com.tzld.longarticle.recommend.server.util.http;

import org.apache.http.Header;

import java.io.UnsupportedEncodingException;

/**
 * HTTP请求响应报文封装类
 * 
 */
public class HttpResponseContent {

	private int statusCode; // 响应状态码
	private String mimeType; // mime类型
	private String charsetName; // 字符集
	private byte[] contentBytes; // 响应报文主体内容的二进制数据
	private Header[] headers; // 响应头

	/**
	 * 获取响应报文主体内容，字符串,使用响应报文中的字符集编码
	 * 
	 * @return
	 */
	public String getBodyContent() {
		return this.getBodyContent(charsetName);
	}

	/**
	 * 获取响应报文主体内容，字符串
	 * 
	 * @param charsetName
	 *            指定的字符编码
	 * @return
	 */
	public String getBodyContent(String charsetName) {
		String str = null;
		if (charsetName == null) {
			charsetName = "UTF-8";
		}
		try {
			str = new String(contentBytes, charsetName);
		} catch (UnsupportedEncodingException e) {
			// 忽略异常
			e.printStackTrace();
		}
		return str;
	}

	public boolean isSuccessful(){
		return statusCode == 200;
	}

	public int getStatusCode() {
		return statusCode;
	}

	public void setStatusCode(int statusCode) {
		this.statusCode = statusCode;
	}

	public String getMimeType() {
		return mimeType;
	}

	public void setMimeType(String mimeType) {
		this.mimeType = mimeType;
	}

	public String getCharsetName() {
		return charsetName;
	}

	public void setCharsetName(String charsetName) {
		this.charsetName = charsetName;
	}

	public byte[] getContentBytes() {
		return contentBytes;
	}

	public void setContentBytes(byte[] contentBytes) {
		this.contentBytes = contentBytes;
	}

	public Header[] getHeaders() {
		return headers;
	}

	public void setHeaders(Header[] headers) {
		this.headers = headers;
	}

	@Override
	public String toString() {
		return "HttpResponseContent [statusCode=" + statusCode + ", mimeType=" + mimeType + ", charsetName=" + charsetName
				+ ", bodyContent=" + getBodyContent() + "]";
	}

}
